﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Graphics.Printing;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Xaml.Printing;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace SfRichTextBoxAdv_Sample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page , INotifyPropertyChanged
    {
        #region Fields
        bool isDocumentLoaded = true;
        bool canPrint = true;
        PrintDocument printDocument = null;
        IPrintDocumentSource printDocumentSource = null;
        Dictionary<int, BitmapImage> pageImages = null;
        #endregion

        #region Properties
        /// <summary>
        /// Determines whether document is loaded.
        /// </summary>
        public bool IsDocumentLoaded
        {
            get
            {
                return isDocumentLoaded;
            }
            set
            {
                isDocumentLoaded = value;
                NotifyPropertyChanged("CanPrint");
            }
        }
        /// <summary>
        /// Determines whether print can be done.
        /// </summary>
        public bool CanPrint
        {
            get
            {
                return canPrint && isDocumentLoaded;
            }
            set
            {
                canPrint = value;
                NotifyPropertyChanged("CanPrint");
            }
        }
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes the instance of MainPage class.
        /// </summary>
        public MainPage()
        {
            this.InitializeComponent();
            DataContext = this;
            // Loads the document.
            Assembly assembly = typeof(MainPage).GetTypeInfo().Assembly;
            Stream stream = assembly.GetManifestResourceStream("SfRichTextBoxAdv_Sample.Assets.GettingStarted.docx");
            richTextBoxAdv.Load(stream, Syncfusion.UI.Xaml.RichTextBoxAdv.FormatType.Docx);
        }
        #endregion

        #region Events
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion

        #region Implementation
        /// <summary>
        /// Notifies when the property changed.
        /// </summary>
        /// <param name="propertyName"></param>
        private void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        #region Override Implementation
        /// <summary>
        /// Called when navigated to.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);
            RegisterForPrinting();
        }
        /// <summary>
        /// Called when navigated from.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            UnregisterPrinting();
        }
        #endregion

        #region Click Events
        /// <summary>
        /// Called on Load Button clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An <see cref="T:Windows.UI.Xaml.RoutedEventArgs">RoutedEventArgs</see> that contains the event data.</param>
        /// <remarks></remarks>
        private async void LoadButton_Click(object sender, RoutedEventArgs e)
        {
            FileOpenPicker fileOpenPicker = new FileOpenPicker();
            fileOpenPicker.FileTypeFilter.Add(".doc");
            fileOpenPicker.FileTypeFilter.Add(".docx");
            fileOpenPicker.FileTypeFilter.Add(".rtf");
            StorageFile stgFile = await fileOpenPicker.PickSingleFileAsync();
            IsDocumentLoaded = false;
            bool isExceptionCaught = false;
            if (stgFile != null)
            {
                try
                {
                    await richTextBoxAdv.LoadAsync(stgFile);
                }
                catch
                {
                    isExceptionCaught = true;
                }
            }
            if (isExceptionCaught)
            {
                MessageDialog message = new MessageDialog("Something went wrong, the input document could not be processed", "Load Failed");
                await message.ShowAsync();
            }
            IsDocumentLoaded = true;
        }
        /// <summary>
        /// Called on save as image button clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An <see cref="T:Windows.UI.Xaml.RoutedEventArgs">RoutedEventArgs</see> that contains the event data.</param>
        /// <remarks></remarks>
        private async void PrintButton_Click(object sender, RoutedEventArgs e)
        {
            CanPrint = false;
            await GetPagesAsImagesAsync();
            await PrintManager.ShowPrintUIAsync();
            CanPrint = true;
        }
        #endregion

        #region Print Implementation
        /// <summary>
        /// Gets the pages as images asynchronous.
        /// </summary>
        async Task<bool> GetPagesAsImagesAsync()
        {
            pageImages = new Dictionary<int, BitmapImage>();
            TaskCompletionSource<bool> tcs = new TaskCompletionSource<bool>();
            int pageCount = richTextBoxAdv.PageCount;
            for (int i = 0; i < pageCount; i++)
            {
                BitmapImage bitmapImage = await richTextBoxAdv.GetPageAsImageAsync(i);
                pageImages.Add(i, bitmapImage);
            }
            tcs.SetResult(true);
            return await tcs.Task;
        }
        /// <summary>
        /// This method registers the app for printing with Windows and sets up the necessary event handlers for the print process.
        /// </summary>
        private void RegisterForPrinting()
        {
            printDocument = new PrintDocument();
            printDocumentSource = printDocument.DocumentSource;
            printDocument.Paginate += PrintDocument_Paginate;
            printDocument.GetPreviewPage += PrintDocument_GetPreviewPage;
            printDocument.AddPages += PrintDocument_AddPages;
            // Create a PrintManager and add a handler for printing initialization.
            PrintManager printMan = PrintManager.GetForCurrentView();
            printMan.PrintTaskRequested += PrintTaskRequested;
        }
        /// <summary>
        /// This method unregisters the app for printing with Windows.
        /// </summary>
        private void UnregisterPrinting()
        {
            if (printDocument == null)
                return;
            printDocument.Paginate -= PrintDocument_Paginate;
            printDocument.GetPreviewPage -= PrintDocument_GetPreviewPage;
            printDocument.AddPages -= PrintDocument_AddPages;

            // Remove the handler for printing initialization.
            PrintManager printMan = PrintManager.GetForCurrentView();
            printMan.PrintTaskRequested -= PrintTaskRequested;
        }
        /// <summary>
        /// Prints the task requested.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="PrintTaskRequestedEventArgs"/> instance containing the event data.</param>
        private void PrintTaskRequested(PrintManager sender, PrintTaskRequestedEventArgs e)
        {
            PrintTask printTask = null;
            printTask = e.Request.CreatePrintTask("SamplePdf", sourceRequested =>
            {
                // Print Task event handler is invoked when the print job is completed.
                printTask.Completed += PrintTask_Completed;
                sourceRequested.SetSource(printDocumentSource);
            });
        }
        /// <summary>
        /// Prints the task completed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="args">The <see cref="PrintTaskCompletedEventArgs"/> instance containing the event data.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        private void PrintTask_Completed(PrintTask sender, PrintTaskCompletedEventArgs args)
        {
            pageImages.Clear();
            pageImages = null;
        }
        /// <summary>
        /// Handles the AddPages event of the printDocument control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="AddPagesEventArgs"/> instance containing the event data.</param>
        void PrintDocument_AddPages(object sender, AddPagesEventArgs e)
        {
            int pageCount = richTextBoxAdv.PageCount;
            for (int i = 0; i < pageCount; i++)
            {
                if (pageImages.ContainsKey(i))
                {
                    Image image = new Image();
                    image.Source = pageImages[i];
                    printDocument.AddPage(image);
                }
            }
            printDocument.AddPagesComplete();
        }
        /// <summary>
        /// Handles the GetPreviewPage event of the printDocument control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="GetPreviewPageEventArgs"/> instance containing the event data.</param>
        void PrintDocument_GetPreviewPage(object sender, GetPreviewPageEventArgs e)
        {
            PrintDocument printDocument = sender as PrintDocument;
            int currentPreviewPage = 0;
            Interlocked.Exchange(ref currentPreviewPage, e.PageNumber - 1);
            if (pageImages.ContainsKey(e.PageNumber - 1))
            {
                BitmapImage bitmap = pageImages[e.PageNumber - 1];
                Image image = new Image();
                image.Source = bitmap;
                printDocument.SetPreviewPage(e.PageNumber, image);
            }
        }
        /// <summary>
        /// Handles the Paginate event of the printDocument control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="PaginateEventArgs"/> instance containing the event data.</param>
        void PrintDocument_Paginate(object sender, PaginateEventArgs e)
        {
            int pageCount = richTextBoxAdv.PageCount;
            PrintDocument printDocument = sender as PrintDocument;
            // Report the number of preview pages created
            printDocument.SetPreviewPageCount(pageCount, PreviewPageCountType.Intermediate);
        }
        #endregion
    }
}
